/* TagPage.c generated by valac 0.32.1, the Vala compiler
 * generated from TagPage.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <gdk/gdk.h>
#include <glib/gi18n-lib.h>


#define TYPE_PAGE (page_get_type ())
#define PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE, Page))
#define PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE, PageClass))
#define IS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE))
#define IS_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE))
#define PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE, PageClass))

typedef struct _Page Page;
typedef struct _PageClass PageClass;
typedef struct _PagePrivate PagePrivate;

#define TYPE_PAGE_WINDOW (page_window_get_type ())
#define PAGE_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE_WINDOW, PageWindow))
#define PAGE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE_WINDOW, PageWindowClass))
#define IS_PAGE_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE_WINDOW))
#define IS_PAGE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE_WINDOW))
#define PAGE_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE_WINDOW, PageWindowClass))

typedef struct _PageWindow PageWindow;
typedef struct _PageWindowClass PageWindowClass;

#define TYPE_FULLSCREEN_WINDOW (fullscreen_window_get_type ())
#define FULLSCREEN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FULLSCREEN_WINDOW, FullscreenWindow))
#define FULLSCREEN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FULLSCREEN_WINDOW, FullscreenWindowClass))
#define IS_FULLSCREEN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FULLSCREEN_WINDOW))
#define IS_FULLSCREEN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FULLSCREEN_WINDOW))
#define FULLSCREEN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FULLSCREEN_WINDOW, FullscreenWindowClass))

typedef struct _FullscreenWindow FullscreenWindow;
typedef struct _FullscreenWindowClass FullscreenWindowClass;

#define TYPE_INJECTION_GROUP (injection_group_get_type ())
#define INJECTION_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_INJECTION_GROUP, InjectionGroup))
#define INJECTION_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_INJECTION_GROUP, InjectionGroupClass))
#define IS_INJECTION_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_INJECTION_GROUP))
#define IS_INJECTION_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_INJECTION_GROUP))
#define INJECTION_GROUP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_INJECTION_GROUP, InjectionGroupClass))

typedef struct _InjectionGroup InjectionGroup;
typedef struct _InjectionGroupClass InjectionGroupClass;

#define TYPE_CHECKERBOARD_PAGE (checkerboard_page_get_type ())
#define CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPage))
#define CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))
#define IS_CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CHECKERBOARD_PAGE))
#define IS_CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CHECKERBOARD_PAGE))
#define CHECKERBOARD_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))

typedef struct _CheckerboardPage CheckerboardPage;
typedef struct _CheckerboardPageClass CheckerboardPageClass;
typedef struct _CheckerboardPagePrivate CheckerboardPagePrivate;

#define TYPE_DATA_OBJECT (data_object_get_type ())
#define DATA_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATA_OBJECT, DataObject))
#define DATA_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATA_OBJECT, DataObjectClass))
#define IS_DATA_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATA_OBJECT))
#define IS_DATA_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATA_OBJECT))
#define DATA_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATA_OBJECT, DataObjectClass))

typedef struct _DataObject DataObject;
typedef struct _DataObjectClass DataObjectClass;

#define TYPE_DATA_VIEW (data_view_get_type ())
#define DATA_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATA_VIEW, DataView))
#define DATA_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATA_VIEW, DataViewClass))
#define IS_DATA_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATA_VIEW))
#define IS_DATA_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATA_VIEW))
#define DATA_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATA_VIEW, DataViewClass))

typedef struct _DataView DataView;
typedef struct _DataViewClass DataViewClass;

#define TYPE_THUMBNAIL_VIEW (thumbnail_view_get_type ())
#define THUMBNAIL_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL_VIEW, ThumbnailView))
#define THUMBNAIL_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL_VIEW, ThumbnailViewClass))
#define IS_THUMBNAIL_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL_VIEW))
#define IS_THUMBNAIL_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL_VIEW))
#define THUMBNAIL_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL_VIEW, ThumbnailViewClass))

typedef struct _ThumbnailView ThumbnailView;
typedef struct _ThumbnailViewClass ThumbnailViewClass;

#define TYPE_CHECKERBOARD_ITEM (checkerboard_item_get_type ())
#define CHECKERBOARD_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CHECKERBOARD_ITEM, CheckerboardItem))
#define CHECKERBOARD_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CHECKERBOARD_ITEM, CheckerboardItemClass))
#define IS_CHECKERBOARD_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CHECKERBOARD_ITEM))
#define IS_CHECKERBOARD_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CHECKERBOARD_ITEM))
#define CHECKERBOARD_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CHECKERBOARD_ITEM, CheckerboardItemClass))

typedef struct _CheckerboardItem CheckerboardItem;
typedef struct _CheckerboardItemClass CheckerboardItemClass;

#define CHECKERBOARD_PAGE_TYPE_ACTIVATOR (checkerboard_page_activator_get_type ())

#define CHECKERBOARD_PAGE_TYPE_KEYBOARD_MODIFIERS (checkerboard_page_keyboard_modifiers_get_type ())
typedef struct _CheckerboardPageKeyboardModifiers CheckerboardPageKeyboardModifiers;

#define TYPE_VIEW_FILTER (view_filter_get_type ())
#define VIEW_FILTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_VIEW_FILTER, ViewFilter))
#define VIEW_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_VIEW_FILTER, ViewFilterClass))
#define IS_VIEW_FILTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_VIEW_FILTER))
#define IS_VIEW_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_VIEW_FILTER))
#define VIEW_FILTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_VIEW_FILTER, ViewFilterClass))

typedef struct _ViewFilter ViewFilter;
typedef struct _ViewFilterClass ViewFilterClass;

#define TYPE_SEARCH_VIEW_FILTER (search_view_filter_get_type ())
#define SEARCH_VIEW_FILTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SEARCH_VIEW_FILTER, SearchViewFilter))
#define SEARCH_VIEW_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SEARCH_VIEW_FILTER, SearchViewFilterClass))
#define IS_SEARCH_VIEW_FILTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SEARCH_VIEW_FILTER))
#define IS_SEARCH_VIEW_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SEARCH_VIEW_FILTER))
#define SEARCH_VIEW_FILTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SEARCH_VIEW_FILTER, SearchViewFilterClass))

typedef struct _SearchViewFilter SearchViewFilter;
typedef struct _SearchViewFilterClass SearchViewFilterClass;

#define CORE_TYPE_TRACKER (core_tracker_get_type ())
#define CORE_TRACKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CORE_TYPE_TRACKER, CoreTracker))
#define CORE_TRACKER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CORE_TYPE_TRACKER, CoreTrackerClass))
#define CORE_IS_TRACKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CORE_TYPE_TRACKER))
#define CORE_IS_TRACKER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CORE_TYPE_TRACKER))
#define CORE_TRACKER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CORE_TYPE_TRACKER, CoreTrackerClass))

typedef struct _CoreTracker CoreTracker;
typedef struct _CoreTrackerClass CoreTrackerClass;

#define CORE_TYPE_VIEW_TRACKER (core_view_tracker_get_type ())
#define CORE_VIEW_TRACKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CORE_TYPE_VIEW_TRACKER, CoreViewTracker))
#define CORE_VIEW_TRACKER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CORE_TYPE_VIEW_TRACKER, CoreViewTrackerClass))
#define CORE_IS_VIEW_TRACKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CORE_TYPE_VIEW_TRACKER))
#define CORE_IS_VIEW_TRACKER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CORE_TYPE_VIEW_TRACKER))
#define CORE_VIEW_TRACKER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CORE_TYPE_VIEW_TRACKER, CoreViewTrackerClass))

typedef struct _CoreViewTracker CoreViewTracker;
typedef struct _CoreViewTrackerClass CoreViewTrackerClass;

#define TYPE_MEDIA_PAGE (media_page_get_type ())
#define MEDIA_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MEDIA_PAGE, MediaPage))
#define MEDIA_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MEDIA_PAGE, MediaPageClass))
#define IS_MEDIA_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MEDIA_PAGE))
#define IS_MEDIA_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MEDIA_PAGE))
#define MEDIA_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MEDIA_PAGE, MediaPageClass))

typedef struct _MediaPage MediaPage;
typedef struct _MediaPageClass MediaPageClass;
typedef struct _MediaPagePrivate MediaPagePrivate;

#define TYPE_RATING (rating_get_type ())

#define TYPE_RAW_DEVELOPER (raw_developer_get_type ())

#define TYPE_DATA_SOURCE (data_source_get_type ())
#define DATA_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATA_SOURCE, DataSource))
#define DATA_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATA_SOURCE, DataSourceClass))
#define IS_DATA_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATA_SOURCE))
#define IS_DATA_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATA_SOURCE))
#define DATA_SOURCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATA_SOURCE, DataSourceClass))

typedef struct _DataSource DataSource;
typedef struct _DataSourceClass DataSourceClass;

#define TYPE_COLLECTION_PAGE (collection_page_get_type ())
#define COLLECTION_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLLECTION_PAGE, CollectionPage))
#define COLLECTION_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLLECTION_PAGE, CollectionPageClass))
#define IS_COLLECTION_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLLECTION_PAGE))
#define IS_COLLECTION_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLLECTION_PAGE))
#define COLLECTION_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLLECTION_PAGE, CollectionPageClass))

typedef struct _CollectionPage CollectionPage;
typedef struct _CollectionPageClass CollectionPageClass;
typedef struct _CollectionPagePrivate CollectionPagePrivate;

#define TYPE_TAG_PAGE (tag_page_get_type ())
#define TAG_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TAG_PAGE, TagPage))
#define TAG_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TAG_PAGE, TagPageClass))
#define IS_TAG_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TAG_PAGE))
#define IS_TAG_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TAG_PAGE))
#define TAG_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TAG_PAGE, TagPageClass))

typedef struct _TagPage TagPage;
typedef struct _TagPageClass TagPageClass;
typedef struct _TagPagePrivate TagPagePrivate;

#define TYPE_TAG (tag_get_type ())
#define TAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TAG, Tag))
#define TAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TAG, TagClass))
#define IS_TAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TAG))
#define IS_TAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TAG))
#define TAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TAG, TagClass))

typedef struct _Tag Tag;
typedef struct _TagClass TagClass;

#define TYPE_DATA_COLLECTION (data_collection_get_type ())
#define DATA_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATA_COLLECTION, DataCollection))
#define DATA_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATA_COLLECTION, DataCollectionClass))
#define IS_DATA_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATA_COLLECTION))
#define IS_DATA_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATA_COLLECTION))
#define DATA_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATA_COLLECTION, DataCollectionClass))

typedef struct _DataCollection DataCollection;
typedef struct _DataCollectionClass DataCollectionClass;

#define TYPE_VIEW_COLLECTION (view_collection_get_type ())
#define VIEW_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_VIEW_COLLECTION, ViewCollection))
#define VIEW_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_VIEW_COLLECTION, ViewCollectionClass))
#define IS_VIEW_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_VIEW_COLLECTION))
#define IS_VIEW_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_VIEW_COLLECTION))
#define VIEW_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_VIEW_COLLECTION, ViewCollectionClass))

typedef struct _ViewCollection ViewCollection;
typedef struct _ViewCollectionClass ViewCollectionClass;
#define _data_collection_unref0(var) ((var == NULL) ? NULL : (var = (data_collection_unref (var), NULL)))

#define TYPE_SOURCE_COLLECTION (source_collection_get_type ())
#define SOURCE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SOURCE_COLLECTION, SourceCollection))
#define SOURCE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SOURCE_COLLECTION, SourceCollectionClass))
#define IS_SOURCE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SOURCE_COLLECTION))
#define IS_SOURCE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SOURCE_COLLECTION))
#define SOURCE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SOURCE_COLLECTION, SourceCollectionClass))

typedef struct _SourceCollection SourceCollection;
typedef struct _SourceCollectionClass SourceCollectionClass;

#define TYPE_DATABASE_SOURCE_COLLECTION (database_source_collection_get_type ())
#define DATABASE_SOURCE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATABASE_SOURCE_COLLECTION, DatabaseSourceCollection))
#define DATABASE_SOURCE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATABASE_SOURCE_COLLECTION, DatabaseSourceCollectionClass))
#define IS_DATABASE_SOURCE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATABASE_SOURCE_COLLECTION))
#define IS_DATABASE_SOURCE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATABASE_SOURCE_COLLECTION))
#define DATABASE_SOURCE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATABASE_SOURCE_COLLECTION, DatabaseSourceCollectionClass))

typedef struct _DatabaseSourceCollection DatabaseSourceCollection;
typedef struct _DatabaseSourceCollectionClass DatabaseSourceCollectionClass;

#define TYPE_CONTAINER_SOURCE_COLLECTION (container_source_collection_get_type ())
#define CONTAINER_SOURCE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONTAINER_SOURCE_COLLECTION, ContainerSourceCollection))
#define CONTAINER_SOURCE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONTAINER_SOURCE_COLLECTION, ContainerSourceCollectionClass))
#define IS_CONTAINER_SOURCE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONTAINER_SOURCE_COLLECTION))
#define IS_CONTAINER_SOURCE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONTAINER_SOURCE_COLLECTION))
#define CONTAINER_SOURCE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONTAINER_SOURCE_COLLECTION, ContainerSourceCollectionClass))

typedef struct _ContainerSourceCollection ContainerSourceCollection;
typedef struct _ContainerSourceCollectionClass ContainerSourceCollectionClass;

#define TYPE_TAG_SOURCE_COLLECTION (tag_source_collection_get_type ())
#define TAG_SOURCE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TAG_SOURCE_COLLECTION, TagSourceCollection))
#define TAG_SOURCE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TAG_SOURCE_COLLECTION, TagSourceCollectionClass))
#define IS_TAG_SOURCE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TAG_SOURCE_COLLECTION))
#define IS_TAG_SOURCE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TAG_SOURCE_COLLECTION))
#define TAG_SOURCE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TAG_SOURCE_COLLECTION, TagSourceCollectionClass))

typedef struct _TagSourceCollection TagSourceCollection;
typedef struct _TagSourceCollectionClass TagSourceCollectionClass;

#define TYPE_ALTERATION (alteration_get_type ())
#define ALTERATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_ALTERATION, Alteration))
#define ALTERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_ALTERATION, AlterationClass))
#define IS_ALTERATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_ALTERATION))
#define IS_ALTERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_ALTERATION))
#define ALTERATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_ALTERATION, AlterationClass))

typedef struct _Alteration Alteration;
typedef struct _AlterationClass AlterationClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

#define TYPE_CONFIGURATION_FACADE (configuration_facade_get_type ())
#define CONFIGURATION_FACADE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONFIGURATION_FACADE, ConfigurationFacade))
#define CONFIGURATION_FACADE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONFIGURATION_FACADE, ConfigurationFacadeClass))
#define IS_CONFIGURATION_FACADE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONFIGURATION_FACADE))
#define IS_CONFIGURATION_FACADE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONFIGURATION_FACADE))
#define CONFIGURATION_FACADE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONFIGURATION_FACADE, ConfigurationFacadeClass))

typedef struct _ConfigurationFacade ConfigurationFacade;
typedef struct _ConfigurationFacadeClass ConfigurationFacadeClass;

#define CONFIG_TYPE_FACADE (config_facade_get_type ())
#define CONFIG_FACADE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CONFIG_TYPE_FACADE, ConfigFacade))
#define CONFIG_FACADE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CONFIG_TYPE_FACADE, ConfigFacadeClass))
#define CONFIG_IS_FACADE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CONFIG_TYPE_FACADE))
#define CONFIG_IS_FACADE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CONFIG_TYPE_FACADE))
#define CONFIG_FACADE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CONFIG_TYPE_FACADE, ConfigFacadeClass))

typedef struct _ConfigFacade ConfigFacade;
typedef struct _ConfigFacadeClass ConfigFacadeClass;

#define TYPE_COMMAND (command_get_type ())
#define COMMAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMMAND, Command))
#define COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMMAND, CommandClass))
#define IS_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMMAND))
#define IS_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMMAND))
#define COMMAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMMAND, CommandClass))

typedef struct _Command Command;
typedef struct _CommandClass CommandClass;

#define TYPE_PAGE_COMMAND (page_command_get_type ())
#define PAGE_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE_COMMAND, PageCommand))
#define PAGE_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE_COMMAND, PageCommandClass))
#define IS_PAGE_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE_COMMAND))
#define IS_PAGE_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE_COMMAND))
#define PAGE_COMMAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE_COMMAND, PageCommandClass))

typedef struct _PageCommand PageCommand;
typedef struct _PageCommandClass PageCommandClass;

#define TYPE_SIMPLE_PROXYABLE_COMMAND (simple_proxyable_command_get_type ())
#define SIMPLE_PROXYABLE_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SIMPLE_PROXYABLE_COMMAND, SimpleProxyableCommand))
#define SIMPLE_PROXYABLE_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SIMPLE_PROXYABLE_COMMAND, SimpleProxyableCommandClass))
#define IS_SIMPLE_PROXYABLE_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SIMPLE_PROXYABLE_COMMAND))
#define IS_SIMPLE_PROXYABLE_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SIMPLE_PROXYABLE_COMMAND))
#define SIMPLE_PROXYABLE_COMMAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SIMPLE_PROXYABLE_COMMAND, SimpleProxyableCommandClass))

typedef struct _SimpleProxyableCommand SimpleProxyableCommand;
typedef struct _SimpleProxyableCommandClass SimpleProxyableCommandClass;

#define TYPE_NEW_CHILD_TAG_COMMAND (new_child_tag_command_get_type ())
#define NEW_CHILD_TAG_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NEW_CHILD_TAG_COMMAND, NewChildTagCommand))
#define NEW_CHILD_TAG_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NEW_CHILD_TAG_COMMAND, NewChildTagCommandClass))
#define IS_NEW_CHILD_TAG_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NEW_CHILD_TAG_COMMAND))
#define IS_NEW_CHILD_TAG_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NEW_CHILD_TAG_COMMAND))
#define NEW_CHILD_TAG_COMMAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NEW_CHILD_TAG_COMMAND, NewChildTagCommandClass))

typedef struct _NewChildTagCommand NewChildTagCommand;
typedef struct _NewChildTagCommandClass NewChildTagCommandClass;

#define TYPE_COMMAND_MANAGER (command_manager_get_type ())
#define COMMAND_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMMAND_MANAGER, CommandManager))
#define COMMAND_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMMAND_MANAGER, CommandManagerClass))
#define IS_COMMAND_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMMAND_MANAGER))
#define IS_COMMAND_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMMAND_MANAGER))
#define COMMAND_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMMAND_MANAGER, CommandManagerClass))

typedef struct _CommandManager CommandManager;
typedef struct _CommandManagerClass CommandManagerClass;
#define _command_manager_unref0(var) ((var == NULL) ? NULL : (var = (command_manager_unref (var), NULL)))

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;

#define TYPE_LIBRARY_WINDOW (library_window_get_type ())
#define LIBRARY_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LIBRARY_WINDOW, LibraryWindow))
#define LIBRARY_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LIBRARY_WINDOW, LibraryWindowClass))
#define IS_LIBRARY_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LIBRARY_WINDOW))
#define IS_LIBRARY_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LIBRARY_WINDOW))
#define LIBRARY_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LIBRARY_WINDOW, LibraryWindowClass))

typedef struct _LibraryWindow LibraryWindow;
typedef struct _LibraryWindowClass LibraryWindowClass;

#define TYPE_DELETE_TAG_COMMAND (delete_tag_command_get_type ())
#define DELETE_TAG_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DELETE_TAG_COMMAND, DeleteTagCommand))
#define DELETE_TAG_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DELETE_TAG_COMMAND, DeleteTagCommandClass))
#define IS_DELETE_TAG_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DELETE_TAG_COMMAND))
#define IS_DELETE_TAG_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DELETE_TAG_COMMAND))
#define DELETE_TAG_COMMAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DELETE_TAG_COMMAND, DeleteTagCommandClass))

typedef struct _DeleteTagCommand DeleteTagCommand;
typedef struct _DeleteTagCommandClass DeleteTagCommandClass;

#define TYPE_THUMBNAIL_SOURCE (thumbnail_source_get_type ())
#define THUMBNAIL_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL_SOURCE, ThumbnailSource))
#define THUMBNAIL_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL_SOURCE, ThumbnailSourceClass))
#define IS_THUMBNAIL_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL_SOURCE))
#define IS_THUMBNAIL_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL_SOURCE))
#define THUMBNAIL_SOURCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL_SOURCE, ThumbnailSourceClass))

typedef struct _ThumbnailSource ThumbnailSource;
typedef struct _ThumbnailSourceClass ThumbnailSourceClass;

#define TYPE_MEDIA_SOURCE (media_source_get_type ())
#define MEDIA_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MEDIA_SOURCE, MediaSource))
#define MEDIA_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MEDIA_SOURCE, MediaSourceClass))
#define IS_MEDIA_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MEDIA_SOURCE))
#define IS_MEDIA_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MEDIA_SOURCE))
#define MEDIA_SOURCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MEDIA_SOURCE, MediaSourceClass))

typedef struct _MediaSource MediaSource;
typedef struct _MediaSourceClass MediaSourceClass;

#define TYPE_TAG_UNTAG_PHOTOS_COMMAND (tag_untag_photos_command_get_type ())
#define TAG_UNTAG_PHOTOS_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TAG_UNTAG_PHOTOS_COMMAND, TagUntagPhotosCommand))
#define TAG_UNTAG_PHOTOS_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TAG_UNTAG_PHOTOS_COMMAND, TagUntagPhotosCommandClass))
#define IS_TAG_UNTAG_PHOTOS_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TAG_UNTAG_PHOTOS_COMMAND))
#define IS_TAG_UNTAG_PHOTOS_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TAG_UNTAG_PHOTOS_COMMAND))
#define TAG_UNTAG_PHOTOS_COMMAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TAG_UNTAG_PHOTOS_COMMAND, TagUntagPhotosCommandClass))

typedef struct _TagUntagPhotosCommand TagUntagPhotosCommand;
typedef struct _TagUntagPhotosCommandClass TagUntagPhotosCommandClass;

struct _Page {
	GtkScrolledWindow parent_instance;
	PagePrivate * priv;
	GtkUIManager* ui;
	GtkToolbar* toolbar;
	gboolean in_view;
};

struct _PageClass {
	GtkScrolledWindowClass parent_class;
	void (*set_page_name) (Page* self, const gchar* page_name);
	void (*set_container) (Page* self, GtkWindow* container);
	void (*clear_container) (Page* self);
	GtkMenuBar* (*get_menubar) (Page* self);
	GtkWidget* (*get_page_ui_widget) (Page* self, const gchar* path);
	GtkToolbar* (*get_toolbar) (Page* self);
	GtkMenu* (*get_page_context_menu) (Page* self);
	void (*switching_from) (Page* self);
	void (*switched_to) (Page* self);
	void (*ready) (Page* self);
	void (*switching_to_fullscreen) (Page* self, FullscreenWindow* fsw);
	void (*returning_from_fullscreen) (Page* self, FullscreenWindow* fsw);
	void (*init_collect_ui_filenames) (Page* self, GeeList* ui_filenames);
	GtkActionEntry* (*init_collect_action_entries) (Page* self, int* result_length1);
	GtkToggleActionEntry* (*init_collect_toggle_action_entries) (Page* self, int* result_length1);
	void (*register_radio_actions) (Page* self, GtkActionGroup* action_group);
	InjectionGroup** (*init_collect_injection_groups) (Page* self, int* result_length1);
	void (*init_actions) (Page* self, gint selected_count, gint count);
	void (*update_actions) (Page* self, gint selected_count, gint count);
	gboolean (*source_drag_failed) (Page* self, GdkDragContext* context, GtkDragResult drag_result);
	gboolean (*on_left_click) (Page* self, GdkEventButton* event);
	gboolean (*on_middle_click) (Page* self, GdkEventButton* event);
	gboolean (*on_right_click) (Page* self, GdkEventButton* event);
	gboolean (*on_left_released) (Page* self, GdkEventButton* event);
	gboolean (*on_middle_released) (Page* self, GdkEventButton* event);
	gboolean (*on_right_released) (Page* self, GdkEventButton* event);
	gboolean (*on_ctrl_pressed) (Page* self, GdkEventKey* event);
	gboolean (*on_ctrl_released) (Page* self, GdkEventKey* event);
	gboolean (*on_alt_pressed) (Page* self, GdkEventKey* event);
	gboolean (*on_alt_released) (Page* self, GdkEventKey* event);
	gboolean (*on_shift_pressed) (Page* self, GdkEventKey* event);
	gboolean (*on_shift_released) (Page* self, GdkEventKey* event);
	gboolean (*on_super_pressed) (Page* self, GdkEventKey* event);
	gboolean (*on_super_released) (Page* self, GdkEventKey* event);
	gboolean (*on_app_key_pressed) (Page* self, GdkEventKey* event);
	gboolean (*on_app_key_released) (Page* self, GdkEventKey* event);
	void (*on_move) (Page* self, GdkRectangle* rect);
	void (*on_move_start) (Page* self, GdkRectangle* rect);
	void (*on_move_finished) (Page* self, GdkRectangle* rect);
	void (*on_resize) (Page* self, GdkRectangle* rect);
	void (*on_resize_start) (Page* self, GdkRectangle* rect);
	void (*on_resize_finished) (Page* self, GdkRectangle* rect);
	gboolean (*on_configure) (Page* self, GdkEventConfigure* event, GdkRectangle* rect);
	gboolean (*on_motion) (Page* self, GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
	gboolean (*on_leave_notify_event) (Page* self);
	gboolean (*on_mousewheel_up) (Page* self, GdkEventScroll* event);
	gboolean (*on_mousewheel_down) (Page* self, GdkEventScroll* event);
	gboolean (*on_mousewheel_left) (Page* self, GdkEventScroll* event);
	gboolean (*on_mousewheel_right) (Page* self, GdkEventScroll* event);
	gboolean (*on_context_keypress) (Page* self);
	gboolean (*on_context_buttonpress) (Page* self, GdkEventButton* event);
	gboolean (*on_context_invoked) (Page* self);
	void (*set_page_cursor) (Page* self, GdkCursorType cursor_type);
};

typedef enum  {
	CHECKERBOARD_PAGE_ACTIVATOR_KEYBOARD,
	CHECKERBOARD_PAGE_ACTIVATOR_MOUSE
} CheckerboardPageActivator;

struct _CheckerboardPageKeyboardModifiers {
	gboolean ctrl_pressed;
	gboolean alt_pressed;
	gboolean shift_pressed;
	gboolean super_pressed;
};

struct _CheckerboardPage {
	Page parent_instance;
	CheckerboardPagePrivate * priv;
	CheckerboardItem* anchor;
	CheckerboardItem* cursor;
};

struct _CheckerboardPageClass {
	PageClass parent_class;
	GtkMenu* (*get_item_context_menu) (CheckerboardPage* self);
	gchar* (*get_view_empty_message) (CheckerboardPage* self);
	gchar* (*get_filter_no_match_message) (CheckerboardPage* self);
	void (*on_item_activated) (CheckerboardPage* self, CheckerboardItem* item, CheckerboardPageActivator activator, CheckerboardPageKeyboardModifiers* modifiers);
	SearchViewFilter* (*get_search_view_filter) (CheckerboardPage* self);
	CoreViewTracker* (*get_view_tracker) (CheckerboardPage* self);
	gboolean (*on_mouse_over) (CheckerboardPage* self, CheckerboardItem* item, gint x, gint y, GdkModifierType mask);
	void (*set_display_titles) (CheckerboardPage* self, gboolean display);
	void (*set_display_comments) (CheckerboardPage* self, gboolean display);
};

typedef enum  {
	RATING_REJECTED = -1,
	RATING_UNRATED = 0,
	RATING_ONE = 1,
	RATING_TWO = 2,
	RATING_THREE = 3,
	RATING_FOUR = 4,
	RATING_FIVE = 5
} Rating;

typedef enum  {
	RAW_DEVELOPER_SHOTWELL = 0,
	RAW_DEVELOPER_CAMERA,
	RAW_DEVELOPER_EMBEDDED
} RawDeveloper;

struct _MediaPage {
	CheckerboardPage parent_instance;
	MediaPagePrivate * priv;
};

struct _MediaPageClass {
	CheckerboardPageClass parent_class;
	void (*on_zoom_changed) (MediaPage* self);
	void (*on_export) (MediaPage* self);
	void (*on_increase_size) (MediaPage* self);
	void (*on_decrease_size) (MediaPage* self);
	void (*on_increase_rating) (MediaPage* self);
	void (*on_decrease_rating) (MediaPage* self);
	void (*on_set_rating) (MediaPage* self, Rating rating);
	void (*on_rate_rejected) (MediaPage* self);
	void (*on_rate_unrated) (MediaPage* self);
	void (*on_rate_one) (MediaPage* self);
	void (*on_rate_two) (MediaPage* self);
	void (*on_rate_three) (MediaPage* self);
	void (*on_rate_four) (MediaPage* self);
	void (*on_rate_five) (MediaPage* self);
	void (*on_move_to_trash) (MediaPage* self);
	void (*on_edit_title) (MediaPage* self);
	void (*on_edit_comment) (MediaPage* self);
	void (*on_display_titles) (MediaPage* self, GtkAction* action);
	void (*on_display_comments) (MediaPage* self, GtkAction* action);
	void (*on_display_ratings) (MediaPage* self, GtkAction* action);
	void (*on_display_tags) (MediaPage* self, GtkAction* action);
	void (*get_config_photos_sort) (MediaPage* self, gboolean* sort_order, gint* sort_by);
	void (*set_config_photos_sort) (MediaPage* self, gboolean sort_order, gint sort_by);
	void (*on_sort_changed) (MediaPage* self);
	void (*developer_changed) (MediaPage* self, RawDeveloper rd);
	DataView* (*create_thumbnail) (MediaPage* self, DataSource* source);
};

struct _CollectionPage {
	MediaPage parent_instance;
	CollectionPagePrivate * priv;
};

struct _CollectionPageClass {
	MediaPageClass parent_class;
};

struct _TagPage {
	CollectionPage parent_instance;
	TagPagePrivate * priv;
};

struct _TagPageClass {
	CollectionPageClass parent_class;
};

struct _TagPagePrivate {
	Tag* tag;
};

typedef DataView* (*CreateView) (DataSource* source, void* user_data);

static gpointer tag_page_parent_class = NULL;
extern TagSourceCollection* tag_global;

GType page_get_type (void) G_GNUC_CONST;
GType page_window_get_type (void) G_GNUC_CONST;
GType fullscreen_window_get_type (void) G_GNUC_CONST;
gpointer injection_group_ref (gpointer instance);
void injection_group_unref (gpointer instance);
GParamSpec* param_spec_injection_group (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_injection_group (GValue* value, gpointer v_object);
void value_take_injection_group (GValue* value, gpointer v_object);
gpointer value_get_injection_group (const GValue* value);
GType injection_group_get_type (void) G_GNUC_CONST;
GType checkerboard_page_get_type (void) G_GNUC_CONST;
GType data_object_get_type (void) G_GNUC_CONST;
GType data_view_get_type (void) G_GNUC_CONST;
GType thumbnail_view_get_type (void) G_GNUC_CONST;
GType checkerboard_item_get_type (void) G_GNUC_CONST;
GType checkerboard_page_activator_get_type (void) G_GNUC_CONST;
GType checkerboard_page_keyboard_modifiers_get_type (void) G_GNUC_CONST;
CheckerboardPageKeyboardModifiers* checkerboard_page_keyboard_modifiers_dup (const CheckerboardPageKeyboardModifiers* self);
void checkerboard_page_keyboard_modifiers_free (CheckerboardPageKeyboardModifiers* self);
gpointer view_filter_ref (gpointer instance);
void view_filter_unref (gpointer instance);
GParamSpec* param_spec_view_filter (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_view_filter (GValue* value, gpointer v_object);
void value_take_view_filter (GValue* value, gpointer v_object);
gpointer value_get_view_filter (const GValue* value);
GType view_filter_get_type (void) G_GNUC_CONST;
GType search_view_filter_get_type (void) G_GNUC_CONST;
gpointer core_tracker_ref (gpointer instance);
void core_tracker_unref (gpointer instance);
GParamSpec* core_param_spec_tracker (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void core_value_set_tracker (GValue* value, gpointer v_object);
void core_value_take_tracker (GValue* value, gpointer v_object);
gpointer core_value_get_tracker (const GValue* value);
GType core_tracker_get_type (void) G_GNUC_CONST;
GType core_view_tracker_get_type (void) G_GNUC_CONST;
GType media_page_get_type (void) G_GNUC_CONST;
GType rating_get_type (void) G_GNUC_CONST;
GType raw_developer_get_type (void) G_GNUC_CONST;
GType data_source_get_type (void) G_GNUC_CONST;
GType collection_page_get_type (void) G_GNUC_CONST;
GType tag_page_get_type (void) G_GNUC_CONST;
GType tag_get_type (void) G_GNUC_CONST;
#define TAG_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_TAG_PAGE, TagPagePrivate))
enum  {
	TAG_PAGE_DUMMY_PROPERTY
};
gpointer data_collection_ref (gpointer instance);
void data_collection_unref (gpointer instance);
GParamSpec* param_spec_data_collection (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_data_collection (GValue* value, gpointer v_object);
void value_take_data_collection (GValue* value, gpointer v_object);
gpointer value_get_data_collection (const GValue* value);
GType data_collection_get_type (void) G_GNUC_CONST;
GType view_collection_get_type (void) G_GNUC_CONST;
ViewCollection* page_get_view (Page* self);
void view_collection_halt_mirroring (ViewCollection* self);
GType source_collection_get_type (void) G_GNUC_CONST;
GType database_source_collection_get_type (void) G_GNUC_CONST;
GType container_source_collection_get_type (void) G_GNUC_CONST;
GType tag_source_collection_get_type (void) G_GNUC_CONST;
gpointer alteration_ref (gpointer instance);
void alteration_unref (gpointer instance);
GParamSpec* param_spec_alteration (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_alteration (GValue* value, gpointer v_object);
void value_take_alteration (GValue* value, gpointer v_object);
gpointer value_get_alteration (const GValue* value);
GType alteration_get_type (void) G_GNUC_CONST;
static void tag_page_on_tags_altered (TagPage* self, GeeMap* map);
static void _tag_page_on_tags_altered_data_collection_items_altered (DataCollection* _sender, GeeMap* items, gpointer self);
TagPage* tag_page_new (Tag* tag);
TagPage* tag_page_construct (GType object_type, Tag* tag);
gchar* data_object_get_name (DataObject* self);
CollectionPage* collection_page_construct (GType object_type, const gchar* page_name);
void tag_mirror_sources (Tag* self, ViewCollection* view, CreateView mirroring_ctor, void* mirroring_ctor_target);
DataView* media_page_create_thumbnail (MediaPage* self, DataSource* source);
static DataView* _media_page_create_thumbnail_create_view (DataSource* source, gpointer self);
void checkerboard_page_init_page_context_menu (CheckerboardPage* self, const gchar* path);
static void tag_page_real_init_collect_ui_filenames (Page* base, GeeList* ui_filenames);
void page_init_collect_ui_filenames (Page* self, GeeList* ui_filenames);
Tag* tag_page_get_tag (TagPage* self);
static void tag_page_real_get_config_photos_sort (MediaPage* base, gboolean* sort_order, gint* sort_by);
GType configuration_facade_get_type (void) G_GNUC_CONST;
GType config_facade_get_type (void) G_GNUC_CONST;
ConfigFacade* config_facade_get_instance (void);
void configuration_facade_get_event_photos_sort (ConfigurationFacade* self, gboolean* sort_order, gint* sort_by);
static void tag_page_real_set_config_photos_sort (MediaPage* base, gboolean sort_order, gint sort_by);
void configuration_facade_set_event_photos_sort (ConfigurationFacade* self, gboolean sort_order, gint sort_by);
static GtkActionEntry* tag_page_real_init_collect_action_entries (Page* base, int* result_length1);
GtkActionEntry* page_init_collect_action_entries (Page* self, int* result_length1);
#define TRANSLATABLE "translatable"
static void tag_page_on_delete_tag (TagPage* self);
static void _tag_page_on_delete_tag_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add134 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
static void tag_page_on_rename_tag (TagPage* self);
static void _tag_page_on_rename_tag_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add135 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
static void tag_page_on_remove_tag_from_photos (TagPage* self);
static void _tag_page_on_remove_tag_from_photos_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add136 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
#define RESOURCES_DELETE_TAG_SIDEBAR_MENU _ ("_Delete")
static void _vala_array_add137 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
#define RESOURCES_RENAME_TAG_SIDEBAR_MENU _ ("_Rename...")
static void _vala_array_add138 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
#define RESOURCES_NEW_CHILD_TAG_SIDEBAR_MENU _ ("_New")
static void tag_page_on_new_child_tag_sidebar (TagPage* self);
static void _tag_page_on_new_child_tag_sidebar_gtk_action_callback (GtkAction* action, gpointer self);
static void _vala_array_add139 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value);
void page_set_page_name (Page* self, const gchar* page_name);
void page_update_actions (Page* self, gint selected_count, gint count);
gint view_collection_get_selected_count (ViewCollection* self);
gint data_collection_get_count (DataCollection* self);
static void tag_page_real_update_actions (Page* base, gint selected_count, gint count);
void page_set_action_details (Page* self, const gchar* name, const gchar* label, const gchar* tooltip, gboolean sensitive);
gchar* resources_delete_tag_menu (const gchar* name);
gchar* tag_get_user_visible_name (Tag* self);
gchar* resources_rename_tag_menu (const gchar* name);
gchar* resources_untag_photos_menu (const gchar* name, gint count);
GType command_get_type (void) G_GNUC_CONST;
GType page_command_get_type (void) G_GNUC_CONST;
GType simple_proxyable_command_get_type (void) G_GNUC_CONST;
GType new_child_tag_command_get_type (void) G_GNUC_CONST;
NewChildTagCommand* new_child_tag_command_new (Tag* tag);
NewChildTagCommand* new_child_tag_command_construct (GType object_type, Tag* tag);
gpointer command_manager_ref (gpointer instance);
void command_manager_unref (gpointer instance);
GParamSpec* param_spec_command_manager (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_command_manager (GValue* value, gpointer v_object);
void value_take_command_manager (GValue* value, gpointer v_object);
gpointer value_get_command_manager (const GValue* value);
GType command_manager_get_type (void) G_GNUC_CONST;
CommandManager* app_window_get_command_manager (void);
void command_manager_execute (CommandManager* self, Command* command);
GType app_window_get_type (void) G_GNUC_CONST;
GType library_window_get_type (void) G_GNUC_CONST;
LibraryWindow* library_window_get_app (void);
void library_window_rename_tag_in_sidebar (LibraryWindow* self, Tag* tag);
Tag* new_child_tag_command_get_created_child (NewChildTagCommand* self);
gboolean dialogs_confirm_delete_tag (Tag* tag);
DeleteTagCommand* delete_tag_command_new (Tag* tag);
DeleteTagCommand* delete_tag_command_construct (GType object_type, Tag* tag);
GType delete_tag_command_get_type (void) G_GNUC_CONST;
CommandManager* page_get_command_manager (Page* self);
GeeList* view_collection_get_selected_sources (ViewCollection* self);
GType thumbnail_source_get_type (void) G_GNUC_CONST;
GType media_source_get_type (void) G_GNUC_CONST;
TagUntagPhotosCommand* tag_untag_photos_command_new (Tag* tag, GeeCollection* sources, gint count, gboolean attach);
TagUntagPhotosCommand* tag_untag_photos_command_construct (GType object_type, Tag* tag, GeeCollection* sources, gint count, gboolean attach);
GType tag_untag_photos_command_get_type (void) G_GNUC_CONST;
static void tag_page_finalize (GObject* obj);


static void _tag_page_on_tags_altered_data_collection_items_altered (DataCollection* _sender, GeeMap* items, gpointer self) {
#line 23 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	tag_page_on_tags_altered ((TagPage*) self, items);
#line 748 "TagPage.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 13 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	return self ? g_object_ref (self) : NULL;
#line 755 "TagPage.c"
}


static DataView* _media_page_create_thumbnail_create_view (DataSource* source, gpointer self) {
	DataView* result;
	result = media_page_create_thumbnail ((MediaPage*) self, source);
#line 16 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	return result;
#line 764 "TagPage.c"
}


TagPage* tag_page_construct (GType object_type, Tag* tag) {
	TagPage * self = NULL;
	Tag* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	Tag* _tmp3_ = NULL;
	Tag* _tmp4_ = NULL;
	TagSourceCollection* _tmp5_ = NULL;
	Tag* _tmp6_ = NULL;
	ViewCollection* _tmp7_ = NULL;
	ViewCollection* _tmp8_ = NULL;
#line 10 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_return_val_if_fail (IS_TAG (tag), NULL);
#line 11 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = tag;
#line 11 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = data_object_get_name (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, TYPE_DATA_OBJECT, DataObject));
#line 11 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_ = _tmp1_;
#line 11 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	self = (TagPage*) collection_page_construct (object_type, _tmp2_);
#line 11 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_free0 (_tmp2_);
#line 13 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp3_ = tag;
#line 13 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp4_ = _g_object_ref0 (_tmp3_);
#line 13 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_object_unref0 (self->priv->tag);
#line 13 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	self->priv->tag = _tmp4_;
#line 15 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp5_ = tag_global;
#line 15 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (_tmp5_, TYPE_DATA_COLLECTION, DataCollection), "items-altered", (GCallback) _tag_page_on_tags_altered_data_collection_items_altered, self, 0);
#line 16 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp6_ = tag;
#line 16 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp7_ = page_get_view (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page));
#line 16 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp8_ = _tmp7_;
#line 16 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	tag_mirror_sources (_tmp6_, _tmp8_, _media_page_create_thumbnail_create_view, G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_MEDIA_PAGE, MediaPage));
#line 16 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_data_collection_unref0 (_tmp8_);
#line 18 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	checkerboard_page_init_page_context_menu (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_CHECKERBOARD_PAGE, CheckerboardPage), "/TagsContextMenu");
#line 10 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	return self;
#line 817 "TagPage.c"
}


TagPage* tag_page_new (Tag* tag) {
#line 10 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	return tag_page_construct (TYPE_TAG_PAGE, tag);
#line 824 "TagPage.c"
}


static void tag_page_real_init_collect_ui_filenames (Page* base, GeeList* ui_filenames) {
	TagPage * self;
	GeeList* _tmp0_ = NULL;
	GeeList* _tmp1_ = NULL;
#line 26 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_TAG_PAGE, TagPage);
#line 26 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_return_if_fail (GEE_IS_LIST (ui_filenames));
#line 27 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = ui_filenames;
#line 27 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	PAGE_CLASS (tag_page_parent_class)->init_collect_ui_filenames (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_COLLECTION_PAGE, CollectionPage), TYPE_PAGE, Page), _tmp0_);
#line 28 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = ui_filenames;
#line 28 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	gee_collection_add (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, GEE_TYPE_COLLECTION, GeeCollection), "tags.ui");
#line 844 "TagPage.c"
}


Tag* tag_page_get_tag (TagPage* self) {
	Tag* result = NULL;
	Tag* _tmp0_ = NULL;
	Tag* _tmp1_ = NULL;
#line 31 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_return_val_if_fail (IS_TAG_PAGE (self), NULL);
#line 32 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = self->priv->tag;
#line 32 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 32 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	result = _tmp1_;
#line 32 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	return result;
#line 862 "TagPage.c"
}


static void tag_page_real_get_config_photos_sort (MediaPage* base, gboolean* sort_order, gint* sort_by) {
	TagPage * self;
	gboolean _vala_sort_order = FALSE;
	gint _vala_sort_by = 0;
	ConfigFacade* _tmp0_ = NULL;
	ConfigFacade* _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	gint _tmp3_ = 0;
#line 35 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_TAG_PAGE, TagPage);
#line 36 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = config_facade_get_instance ();
#line 36 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = _tmp0_;
#line 36 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	configuration_facade_get_event_photos_sort (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, TYPE_CONFIGURATION_FACADE, ConfigurationFacade), &_tmp2_, &_tmp3_);
#line 36 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_vala_sort_order = _tmp2_;
#line 36 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_vala_sort_by = _tmp3_;
#line 36 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_object_unref0 (_tmp1_);
#line 35 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if (sort_order) {
#line 35 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*sort_order = _vala_sort_order;
#line 892 "TagPage.c"
	}
#line 35 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if (sort_by) {
#line 35 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*sort_by = _vala_sort_by;
#line 898 "TagPage.c"
	}
}


static void tag_page_real_set_config_photos_sort (MediaPage* base, gboolean sort_order, gint sort_by) {
	TagPage * self;
	ConfigFacade* _tmp0_ = NULL;
	ConfigFacade* _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	gint _tmp3_ = 0;
#line 39 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_TAG_PAGE, TagPage);
#line 40 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = config_facade_get_instance ();
#line 40 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = _tmp0_;
#line 40 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_ = sort_order;
#line 40 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp3_ = sort_by;
#line 40 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	configuration_facade_set_event_photos_sort (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, TYPE_CONFIGURATION_FACADE, ConfigurationFacade), _tmp2_, _tmp3_);
#line 40 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_object_unref0 (_tmp1_);
#line 923 "TagPage.c"
}


static void _tag_page_on_delete_tag_gtk_action_callback (GtkAction* action, gpointer self) {
#line 46 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	tag_page_on_delete_tag ((TagPage*) self);
#line 930 "TagPage.c"
}


static void _vala_array_add134 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 48 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if ((*length) == (*size)) {
#line 48 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 48 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 941 "TagPage.c"
	}
#line 48 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	(*array)[(*length)++] = *value;
#line 945 "TagPage.c"
}


static void _tag_page_on_rename_tag_gtk_action_callback (GtkAction* action, gpointer self) {
#line 50 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	tag_page_on_rename_tag ((TagPage*) self);
#line 952 "TagPage.c"
}


static void _vala_array_add135 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 52 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if ((*length) == (*size)) {
#line 52 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 52 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 963 "TagPage.c"
	}
#line 52 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	(*array)[(*length)++] = *value;
#line 967 "TagPage.c"
}


static void _tag_page_on_remove_tag_from_photos_gtk_action_callback (GtkAction* action, gpointer self) {
#line 54 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	tag_page_on_remove_tag_from_photos ((TagPage*) self);
#line 974 "TagPage.c"
}


static void _vala_array_add136 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 57 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if ((*length) == (*size)) {
#line 57 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 57 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 985 "TagPage.c"
	}
#line 57 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	(*array)[(*length)++] = *value;
#line 989 "TagPage.c"
}


static void _vala_array_add137 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 61 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if ((*length) == (*size)) {
#line 61 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 61 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 1000 "TagPage.c"
	}
#line 61 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	(*array)[(*length)++] = *value;
#line 1004 "TagPage.c"
}


static void _vala_array_add138 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 65 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if ((*length) == (*size)) {
#line 65 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 65 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 1015 "TagPage.c"
	}
#line 65 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	(*array)[(*length)++] = *value;
#line 1019 "TagPage.c"
}


static void _tag_page_on_new_child_tag_sidebar_gtk_action_callback (GtkAction* action, gpointer self) {
#line 67 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	tag_page_on_new_child_tag_sidebar ((TagPage*) self);
#line 1026 "TagPage.c"
}


static void _vala_array_add139 (GtkActionEntry** array, int* length, int* size, const GtkActionEntry* value) {
#line 69 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if ((*length) == (*size)) {
#line 69 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 69 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*array = g_renew (GtkActionEntry, *array, *size);
#line 1037 "TagPage.c"
	}
#line 69 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	(*array)[(*length)++] = *value;
#line 1041 "TagPage.c"
}


static GtkActionEntry* tag_page_real_init_collect_action_entries (Page* base, int* result_length1) {
	TagPage * self;
	GtkActionEntry* result = NULL;
	GtkActionEntry* actions = NULL;
	gint _tmp0_ = 0;
	GtkActionEntry* _tmp1_ = NULL;
	gint actions_length1 = 0;
	gint _actions_size_ = 0;
	GtkActionEntry delete_tag = {0};
	GtkActionEntry _tmp2_ = {0};
	GtkActionEntry* _tmp3_ = NULL;
	gint _tmp3__length1 = 0;
	GtkActionEntry _tmp4_ = {0};
	GtkActionEntry rename_tag = {0};
	GtkActionEntry _tmp5_ = {0};
	GtkActionEntry* _tmp6_ = NULL;
	gint _tmp6__length1 = 0;
	GtkActionEntry _tmp7_ = {0};
	GtkActionEntry remove_tag = {0};
	GtkActionEntry _tmp8_ = {0};
	GtkActionEntry* _tmp9_ = NULL;
	gint _tmp9__length1 = 0;
	GtkActionEntry _tmp10_ = {0};
	GtkActionEntry delete_tag_sidebar = {0};
	GtkActionEntry _tmp11_ = {0};
	GtkActionEntry* _tmp12_ = NULL;
	gint _tmp12__length1 = 0;
	GtkActionEntry _tmp13_ = {0};
	GtkActionEntry rename_tag_sidebar = {0};
	GtkActionEntry _tmp14_ = {0};
	GtkActionEntry* _tmp15_ = NULL;
	gint _tmp15__length1 = 0;
	GtkActionEntry _tmp16_ = {0};
	GtkActionEntry new_child_tag_sidebar = {0};
	GtkActionEntry _tmp17_ = {0};
	GtkActionEntry* _tmp18_ = NULL;
	gint _tmp18__length1 = 0;
	GtkActionEntry _tmp19_ = {0};
	GtkActionEntry* _tmp20_ = NULL;
	gint _tmp20__length1 = 0;
#line 43 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_TAG_PAGE, TagPage);
#line 44 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = PAGE_CLASS (tag_page_parent_class)->init_collect_action_entries (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_COLLECTION_PAGE, CollectionPage), TYPE_PAGE, Page), &_tmp0_);
#line 44 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	actions = _tmp1_;
#line 44 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	actions_length1 = _tmp0_;
#line 44 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_actions_size_ = actions_length1;
#line 46 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_.name = "DeleteTag";
#line 46 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_.stock_id = NULL;
#line 46 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_.label = TRANSLATABLE;
#line 46 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_.accelerator = NULL;
#line 46 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_.tooltip = NULL;
#line 46 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_.callback = (GCallback) _tag_page_on_delete_tag_gtk_action_callback;
#line 46 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	delete_tag = _tmp2_;
#line 48 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp3_ = actions;
#line 48 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp3__length1 = actions_length1;
#line 48 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp4_ = delete_tag;
#line 48 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_vala_array_add134 (&actions, &actions_length1, &_actions_size_, &_tmp4_);
#line 50 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp5_.name = "RenameTag";
#line 50 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp5_.stock_id = NULL;
#line 50 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp5_.label = TRANSLATABLE;
#line 50 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp5_.accelerator = NULL;
#line 50 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp5_.tooltip = NULL;
#line 50 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp5_.callback = (GCallback) _tag_page_on_rename_tag_gtk_action_callback;
#line 50 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	rename_tag = _tmp5_;
#line 52 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp6_ = actions;
#line 52 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp6__length1 = actions_length1;
#line 52 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp7_ = rename_tag;
#line 52 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_vala_array_add135 (&actions, &actions_length1, &_actions_size_, &_tmp7_);
#line 54 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp8_.name = "RemoveTagFromPhotos";
#line 54 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp8_.stock_id = NULL;
#line 54 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp8_.label = TRANSLATABLE;
#line 54 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp8_.accelerator = NULL;
#line 54 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp8_.tooltip = NULL;
#line 54 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp8_.callback = (GCallback) _tag_page_on_remove_tag_from_photos_gtk_action_callback;
#line 54 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	remove_tag = _tmp8_;
#line 57 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp9_ = actions;
#line 57 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp9__length1 = actions_length1;
#line 57 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp10_ = remove_tag;
#line 57 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_vala_array_add136 (&actions, &actions_length1, &_actions_size_, &_tmp10_);
#line 59 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp11_.name = "DeleteTagSidebar";
#line 59 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp11_.stock_id = NULL;
#line 59 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp11_.label = RESOURCES_DELETE_TAG_SIDEBAR_MENU;
#line 59 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp11_.accelerator = NULL;
#line 59 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp11_.tooltip = NULL;
#line 59 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp11_.callback = (GCallback) _tag_page_on_delete_tag_gtk_action_callback;
#line 59 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	delete_tag_sidebar = _tmp11_;
#line 61 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp12_ = actions;
#line 61 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp12__length1 = actions_length1;
#line 61 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp13_ = delete_tag_sidebar;
#line 61 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_vala_array_add137 (&actions, &actions_length1, &_actions_size_, &_tmp13_);
#line 63 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp14_.name = "RenameTagSidebar";
#line 63 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp14_.stock_id = NULL;
#line 63 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp14_.label = RESOURCES_RENAME_TAG_SIDEBAR_MENU;
#line 63 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp14_.accelerator = NULL;
#line 63 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp14_.tooltip = NULL;
#line 63 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp14_.callback = (GCallback) _tag_page_on_rename_tag_gtk_action_callback;
#line 63 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	rename_tag_sidebar = _tmp14_;
#line 65 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp15_ = actions;
#line 65 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp15__length1 = actions_length1;
#line 65 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp16_ = rename_tag_sidebar;
#line 65 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_vala_array_add138 (&actions, &actions_length1, &_actions_size_, &_tmp16_);
#line 67 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp17_.name = "NewChildTagSidebar";
#line 67 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp17_.stock_id = NULL;
#line 67 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp17_.label = RESOURCES_NEW_CHILD_TAG_SIDEBAR_MENU;
#line 67 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp17_.accelerator = NULL;
#line 67 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp17_.tooltip = NULL;
#line 67 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp17_.callback = (GCallback) _tag_page_on_new_child_tag_sidebar_gtk_action_callback;
#line 67 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	new_child_tag_sidebar = _tmp17_;
#line 69 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp18_ = actions;
#line 69 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp18__length1 = actions_length1;
#line 69 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp19_ = new_child_tag_sidebar;
#line 69 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_vala_array_add139 (&actions, &actions_length1, &_actions_size_, &_tmp19_);
#line 71 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp20_ = actions;
#line 71 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp20__length1 = actions_length1;
#line 71 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if (result_length1) {
#line 71 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		*result_length1 = _tmp20__length1;
#line 1235 "TagPage.c"
	}
#line 71 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	result = _tmp20_;
#line 71 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	return result;
#line 1241 "TagPage.c"
}


static void tag_page_on_tags_altered (TagPage* self, GeeMap* map) {
	GeeMap* _tmp0_ = NULL;
	Tag* _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
#line 74 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_return_if_fail (IS_TAG_PAGE (self));
#line 74 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_return_if_fail (GEE_IS_MAP (map));
#line 75 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = map;
#line 75 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = self->priv->tag;
#line 75 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_ = gee_map_has_key (_tmp0_, G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, TYPE_DATA_OBJECT, DataObject));
#line 75 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if (_tmp2_) {
#line 1261 "TagPage.c"
		Tag* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		ViewCollection* _tmp6_ = NULL;
		ViewCollection* _tmp7_ = NULL;
		gint _tmp8_ = 0;
		ViewCollection* _tmp9_ = NULL;
		ViewCollection* _tmp10_ = NULL;
		gint _tmp11_ = 0;
#line 76 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp3_ = self->priv->tag;
#line 76 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp4_ = data_object_get_name (G_TYPE_CHECK_INSTANCE_CAST (_tmp3_, TYPE_DATA_OBJECT, DataObject));
#line 76 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp5_ = _tmp4_;
#line 76 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		page_set_page_name (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page), _tmp5_);
#line 76 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_g_free0 (_tmp5_);
#line 77 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp6_ = page_get_view (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page));
#line 77 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp7_ = _tmp6_;
#line 77 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp8_ = view_collection_get_selected_count (_tmp7_);
#line 77 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp9_ = page_get_view (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page));
#line 77 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp10_ = _tmp9_;
#line 77 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp11_ = data_collection_get_count (G_TYPE_CHECK_INSTANCE_CAST (_tmp10_, TYPE_DATA_COLLECTION, DataCollection));
#line 77 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		page_update_actions (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page), _tmp8_, _tmp11_);
#line 77 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_data_collection_unref0 (_tmp10_);
#line 77 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_data_collection_unref0 (_tmp7_);
#line 1299 "TagPage.c"
	}
}


static void tag_page_real_update_actions (Page* base, gint selected_count, gint count) {
	TagPage * self;
	Tag* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	Tag* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	gchar* _tmp9_ = NULL;
	Tag* _tmp10_ = NULL;
	gchar* _tmp11_ = NULL;
	gchar* _tmp12_ = NULL;
	gint _tmp13_ = 0;
	gchar* _tmp14_ = NULL;
	gchar* _tmp15_ = NULL;
	gint _tmp16_ = 0;
	gint _tmp17_ = 0;
	gint _tmp18_ = 0;
#line 81 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (base, TYPE_TAG_PAGE, TagPage);
#line 82 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = self->priv->tag;
#line 82 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = tag_get_user_visible_name (_tmp0_);
#line 82 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_ = _tmp1_;
#line 82 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp3_ = resources_delete_tag_menu (_tmp2_);
#line 82 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp4_ = _tmp3_;
#line 82 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	page_set_action_details (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page), "DeleteTag", _tmp4_, NULL, TRUE);
#line 82 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_free0 (_tmp4_);
#line 82 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_free0 (_tmp2_);
#line 87 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp5_ = self->priv->tag;
#line 87 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp6_ = tag_get_user_visible_name (_tmp5_);
#line 87 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp7_ = _tmp6_;
#line 87 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp8_ = resources_rename_tag_menu (_tmp7_);
#line 87 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp9_ = _tmp8_;
#line 87 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	page_set_action_details (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page), "RenameTag", _tmp9_, NULL, TRUE);
#line 87 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_free0 (_tmp9_);
#line 87 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_free0 (_tmp7_);
#line 92 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp10_ = self->priv->tag;
#line 92 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp11_ = tag_get_user_visible_name (_tmp10_);
#line 92 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp12_ = _tmp11_;
#line 92 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp13_ = selected_count;
#line 92 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp14_ = resources_untag_photos_menu (_tmp12_, _tmp13_);
#line 92 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp15_ = _tmp14_;
#line 92 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp16_ = selected_count;
#line 92 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	page_set_action_details (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page), "RemoveTagFromPhotos", _tmp15_, NULL, _tmp16_ > 0);
#line 92 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_free0 (_tmp15_);
#line 92 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_free0 (_tmp12_);
#line 97 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp17_ = selected_count;
#line 97 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp18_ = count;
#line 97 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	PAGE_CLASS (tag_page_parent_class)->update_actions (G_TYPE_CHECK_INSTANCE_CAST (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_COLLECTION_PAGE, CollectionPage), TYPE_PAGE, Page), _tmp17_, _tmp18_);
#line 1385 "TagPage.c"
}


static void tag_page_on_new_child_tag_sidebar (TagPage* self) {
	NewChildTagCommand* creation_command = NULL;
	Tag* _tmp0_ = NULL;
	NewChildTagCommand* _tmp1_ = NULL;
	CommandManager* _tmp2_ = NULL;
	CommandManager* _tmp3_ = NULL;
	LibraryWindow* _tmp4_ = NULL;
	LibraryWindow* _tmp5_ = NULL;
	Tag* _tmp6_ = NULL;
	Tag* _tmp7_ = NULL;
#line 100 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_return_if_fail (IS_TAG_PAGE (self));
#line 101 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = self->priv->tag;
#line 101 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = new_child_tag_command_new (_tmp0_);
#line 101 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	creation_command = _tmp1_;
#line 103 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_ = app_window_get_command_manager ();
#line 103 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp3_ = _tmp2_;
#line 103 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	command_manager_execute (_tmp3_, G_TYPE_CHECK_INSTANCE_CAST (creation_command, TYPE_COMMAND, Command));
#line 103 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_command_manager_unref0 (_tmp3_);
#line 105 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp4_ = library_window_get_app ();
#line 105 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp5_ = _tmp4_;
#line 105 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp6_ = new_child_tag_command_get_created_child (creation_command);
#line 105 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp7_ = _tmp6_;
#line 105 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	library_window_rename_tag_in_sidebar (_tmp5_, _tmp7_);
#line 105 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_object_unref0 (_tmp7_);
#line 105 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_object_unref0 (_tmp5_);
#line 100 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_object_unref0 (creation_command);
#line 1431 "TagPage.c"
}


static void tag_page_on_rename_tag (TagPage* self) {
	LibraryWindow* _tmp0_ = NULL;
	LibraryWindow* _tmp1_ = NULL;
	Tag* _tmp2_ = NULL;
#line 108 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_return_if_fail (IS_TAG_PAGE (self));
#line 109 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = library_window_get_app ();
#line 109 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = _tmp0_;
#line 109 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_ = self->priv->tag;
#line 109 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	library_window_rename_tag_in_sidebar (_tmp1_, _tmp2_);
#line 109 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_object_unref0 (_tmp1_);
#line 1451 "TagPage.c"
}


static void tag_page_on_delete_tag (TagPage* self) {
	Tag* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
#line 112 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_return_if_fail (IS_TAG_PAGE (self));
#line 113 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = self->priv->tag;
#line 113 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = dialogs_confirm_delete_tag (_tmp0_);
#line 113 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if (_tmp1_) {
#line 1466 "TagPage.c"
		CommandManager* _tmp2_ = NULL;
		CommandManager* _tmp3_ = NULL;
		Tag* _tmp4_ = NULL;
		DeleteTagCommand* _tmp5_ = NULL;
		DeleteTagCommand* _tmp6_ = NULL;
#line 114 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp2_ = app_window_get_command_manager ();
#line 114 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp3_ = _tmp2_;
#line 114 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp4_ = self->priv->tag;
#line 114 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp5_ = delete_tag_command_new (_tmp4_);
#line 114 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp6_ = _tmp5_;
#line 114 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		command_manager_execute (_tmp3_, G_TYPE_CHECK_INSTANCE_CAST (_tmp6_, TYPE_COMMAND, Command));
#line 114 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_g_object_unref0 (_tmp6_);
#line 114 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_command_manager_unref0 (_tmp3_);
#line 1488 "TagPage.c"
	}
}


static void tag_page_on_remove_tag_from_photos (TagPage* self) {
	ViewCollection* _tmp0_ = NULL;
	ViewCollection* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gboolean _tmp3_ = FALSE;
#line 117 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_return_if_fail (IS_TAG_PAGE (self));
#line 118 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = page_get_view (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page));
#line 118 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = _tmp0_;
#line 118 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_ = view_collection_get_selected_count (_tmp1_);
#line 118 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp3_ = _tmp2_ > 0;
#line 118 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_data_collection_unref0 (_tmp1_);
#line 118 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	if (_tmp3_) {
#line 1512 "TagPage.c"
		CommandManager* _tmp4_ = NULL;
		CommandManager* _tmp5_ = NULL;
		Tag* _tmp6_ = NULL;
		ViewCollection* _tmp7_ = NULL;
		ViewCollection* _tmp8_ = NULL;
		GeeList* _tmp9_ = NULL;
		GeeCollection* _tmp10_ = NULL;
		ViewCollection* _tmp11_ = NULL;
		ViewCollection* _tmp12_ = NULL;
		gint _tmp13_ = 0;
		TagUntagPhotosCommand* _tmp14_ = NULL;
		TagUntagPhotosCommand* _tmp15_ = NULL;
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp4_ = page_get_command_manager (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page));
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp5_ = _tmp4_;
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp6_ = self->priv->tag;
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp7_ = page_get_view (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page));
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp8_ = _tmp7_;
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp9_ = view_collection_get_selected_sources (_tmp8_);
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp10_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp9_, GEE_TYPE_COLLECTION, GeeCollection);
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp11_ = page_get_view (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page));
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp12_ = _tmp11_;
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp13_ = view_collection_get_selected_count (_tmp12_);
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp14_ = tag_untag_photos_command_new (_tmp6_, _tmp10_, _tmp13_, FALSE);
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_tmp15_ = _tmp14_;
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		command_manager_execute (_tmp5_, G_TYPE_CHECK_INSTANCE_CAST (_tmp15_, TYPE_COMMAND, Command));
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_g_object_unref0 (_tmp15_);
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_data_collection_unref0 (_tmp12_);
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_g_object_unref0 (_tmp10_);
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_data_collection_unref0 (_tmp8_);
#line 119 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
		_command_manager_unref0 (_tmp5_);
#line 1561 "TagPage.c"
	}
}


static void tag_page_class_init (TagPageClass * klass) {
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	tag_page_parent_class = g_type_class_peek_parent (klass);
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_type_class_add_private (klass, sizeof (TagPagePrivate));
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	((PageClass *) klass)->init_collect_ui_filenames = tag_page_real_init_collect_ui_filenames;
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	((MediaPageClass *) klass)->get_config_photos_sort = tag_page_real_get_config_photos_sort;
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	((MediaPageClass *) klass)->set_config_photos_sort = tag_page_real_set_config_photos_sort;
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	((PageClass *) klass)->init_collect_action_entries = tag_page_real_init_collect_action_entries;
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	((PageClass *) klass)->update_actions = tag_page_real_update_actions;
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	G_OBJECT_CLASS (klass)->finalize = tag_page_finalize;
#line 1583 "TagPage.c"
}


static void tag_page_instance_init (TagPage * self) {
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	self->priv = TAG_PAGE_GET_PRIVATE (self);
#line 1590 "TagPage.c"
}


static void tag_page_finalize (GObject* obj) {
	TagPage * self;
	ViewCollection* _tmp0_ = NULL;
	ViewCollection* _tmp1_ = NULL;
	TagSourceCollection* _tmp2_ = NULL;
	guint _tmp3_ = 0U;
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_TAG_PAGE, TagPage);
#line 22 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp0_ = page_get_view (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_PAGE, Page));
#line 22 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp1_ = _tmp0_;
#line 22 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	view_collection_halt_mirroring (_tmp1_);
#line 22 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_data_collection_unref0 (_tmp1_);
#line 23 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_tmp2_ = tag_global;
#line 23 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_signal_parse_name ("items-altered", TYPE_DATA_COLLECTION, &_tmp3_, NULL, FALSE);
#line 23 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	g_signal_handlers_disconnect_matched (G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, TYPE_DATA_COLLECTION, DataCollection), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _tag_page_on_tags_altered_data_collection_items_altered, self);
#line 8 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	_g_object_unref0 (self->priv->tag);
#line 7 "/home/jens/Source/shotwell/src/tags/TagPage.vala"
	G_OBJECT_CLASS (tag_page_parent_class)->finalize (obj);
#line 1620 "TagPage.c"
}


GType tag_page_get_type (void) {
	static volatile gsize tag_page_type_id__volatile = 0;
	if (g_once_init_enter (&tag_page_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (TagPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tag_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TagPage), 0, (GInstanceInitFunc) tag_page_instance_init, NULL };
		GType tag_page_type_id;
		tag_page_type_id = g_type_register_static (TYPE_COLLECTION_PAGE, "TagPage", &g_define_type_info, 0);
		g_once_init_leave (&tag_page_type_id__volatile, tag_page_type_id);
	}
	return tag_page_type_id__volatile;
}



